import 'dart:ui';

import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_feature.dart';
import 'package:fis_measure/interfaces/process/visuals/visual_area.dart';
import 'package:fis_measure/utils/canvas.dart';
import 'package:fis_measure/values/colors.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/painting.dart';
import 'package:flutter/widgets.dart';
import 'package:get/get.dart';

abstract class MeasureItemFeature implements IMeasureItemFeature {
  late IMeasureItem _refItem;
  late List<DPoint> _innerPoints;
  IVisualArea? _hostVisualArea;
  bool _isActive = true;
  late final int _id;

  @protected
  final paintPan = Paint()
    ..color = const Color.fromARGB(255, 255, 255, 0)
    ..isAntiAlias = true
    ..strokeWidth = 2
    ..style = PaintingStyle.stroke;

  MeasureItemFeature(IMeasureItem refItem) {
    _refItem = refItem;
    _id = refItem.assignId();
    _innerPoints = [];
  }
  @override
  List<DPoint> get innerPoints => _innerPoints;

  @override
  IMeasureItem get refItem => _refItem;

  ///   所在区域
  IVisualArea? get hostVisualArea => _hostVisualArea;
  set hostVisualArea(IVisualArea? value) {
    if (value != _hostVisualArea) {
      _hostVisualArea = value;
    }
  }

  @override
  bool get isActive => _isActive;
  set isActive(bool value) {
    if (value != _isActive) {
      _isActive = value;
    }
  }

  @override
  int get id => _id;

  /// 顶点尺寸
  double get vertexSize => 10.0 * refItem.scaleRatio;

  @protected
  DPoint convert2ViewPoint(Size size, DPoint logicPoint) {
    final x = size.width * logicPoint.x;
    final y = size.height * logicPoint.y;
    return DPoint(x, y);
  }

  /// 画序号
  ///
  /// [text] 自定义序号内容
  @protected
  void drawId(Canvas canvas, Size size, [String? text]) {
    final displayText = text ?? id.toString();
    final fontSize = 14.0 * refItem.scaleRatio; // TODO: from config
    final letterSpacing = 0.0 * refItem.scaleRatio;
    final style = TextStyle(
      fontSize: fontSize,
      color: MeasureColors.Primary,
      // fontWeight: FontWeight.bold,
      letterSpacing: letterSpacing,
    );

    final point = innerPoints[0];
    // TODO : fix bug

    final width = boundingTextSize(displayText, style).width;
    // final width = fontSize * displayText.length +
    //     letterSpacing * (displayText.length - 1);
    final vertexOffsetW = vertexSize / 2;
    var offset = convert2ViewPoint(size, point).toOffset();
    if (offset.dx < width) {
      offset = offset.translate(fontSize + vertexOffsetW, -fontSize);
    } else {
      offset = offset.translate(-width - vertexOffsetW, -fontSize);
    }

    canvas.drawText(
      displayText,
      offset,
      style: style,
    );
  }

  /// 画顶点
  void drawVertex(Canvas canvas, Offset offset, [bool active = false]) {
    canvas.drawVertex(offset, vertexSize, active: active);
  }

  /// 计算文本长度
  static Size boundingTextSize(
    String text,
    TextStyle style, {
    int maxLines = 2 ^ 31,
    double maxWidth = double.infinity,
  }) {
    // if (Get.context == null) return Size.zero;
    if (text.isEmpty) return Size.zero;

    final TextPainter textPainter = TextPainter(
      textDirection: TextDirection.ltr,
      // locale: Localizations.localeOf(Get.context!),
      text: TextSpan(text: text, style: style),
      maxLines: maxLines,
    )..layout(maxWidth: maxWidth);
    return textPainter.size;
  }
}
