import 'dart:typed_data';

import 'package:fis_common/event/event_type.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';

class MeasureImageData {
  final String? patientCode;
  final String? remedicalCode;
  final String? recordCode;
  MeasureImageData({
    this.patientCode,
    this.recordCode,
    this.remedicalCode,
  });
}

/// 图片信息数据
abstract class IMeasureDataController {
  late FEventHandler<String> aiResultsInfoChanged;
  late FEventHandler<String> measureApplicationVersionChanged;
  late FEventHandler<String> measureLanguageChanged;
  late FEventHandler<DiagnosisConclusionEnum> diagnosisConclusionChanged;
  late FEventHandler<List<RemedicalInfoDTO>> remedicalListResultChanged;
  late FEventHandler<String> itemCurrentImageChanged;
  late FEventHandler<List<ItemMetaDTO>> itemMetaListChanged;
  late FEventHandler<List<String>> getMeasureApplicationChanged;
  late FEventHandler<MeasureImageData> measureImageDataChanged;

  /// 该图像下面的应用
  List<ItemMetaDTO> get itemMetaList;
  set itemMetaList(List<ItemMetaDTO> value);

  /// 测量的翻译
  String get measureLanguage;
  set measureLanguage(String value);

  /// 测量应用的版本号
  String get measureApplicationVersion;
  set measureApplicationVersion(String value);

  ///当前选择图像
  String get itemCurrentImage;
  set itemCurrentImage(String value);

  /// 测量图片的ai结果
  String get aiResults;
  set aiResults(String value);

  /// 良恶性 用来判断是否存在ai结果
  DiagnosisConclusionEnum get diagnosisConclusion;
  set diagnosisConclusion(DiagnosisConclusionEnum value);

  /// 测量图片集
  List<RemedicalInfoDTO> get remedicalList;
  set remedicalList(List<RemedicalInfoDTO> value);

  /// 测量方式集
  List<String> get getMeasureApplicationList;
  set getMeasureApplicationList(List<String> value);

  /// 测量图片数据
  MeasureImageData get measureImageData;
  set measureImageData(MeasureImageData value);
}

class MeasureDataController implements IMeasureDataController {
  String _aiResults = '';
  String _itemCurrentImage = '';
  String _measureLanguage = '';
  String _measureApplicationVersion = '';
  List<RemedicalInfoDTO> _remedicalList = [];
  List<ItemMetaDTO> _itemMetaList = [];
  List<String> _getMeasureApplicationList = [];
  DiagnosisConclusionEnum _diagnosisConclusion =
      DiagnosisConclusionEnum.NotRequired;
  MeasureImageData _measureImageData = MeasureImageData();

  @override
  var remedicalListResultChanged = FEventHandler<List<RemedicalInfoDTO>>();

  @override
  var aiResultsInfoChanged = FEventHandler<String>();

  @override
  var measureLanguageChanged = FEventHandler<String>();

  @override
  var measureApplicationVersionChanged = FEventHandler<String>();

  @override
  var diagnosisConclusionChanged = FEventHandler<DiagnosisConclusionEnum>();

  @override
  var itemMetaListChanged = FEventHandler<List<ItemMetaDTO>>();

  @override
  var itemCurrentImageChanged = FEventHandler<String>();

  @override
  var getMeasureApplicationChanged = FEventHandler<List<String>>();

  @override
  var measureImageDataChanged = FEventHandler<MeasureImageData>();

  @override
  List<String> get getMeasureApplicationList => _getMeasureApplicationList;
  @override
  set getMeasureApplicationList(List<String> value) {
    if (value != _getMeasureApplicationList) {
      _getMeasureApplicationList = value;
      _onGetMeasureApplicationChanged();
    }
  }

  @override
  List<ItemMetaDTO> get itemMetaList => _itemMetaList;
  @override
  set itemMetaList(List<ItemMetaDTO> value) {
    if (value != _itemMetaList) {
      _itemMetaList = value;
      _onItemMetaListChanged();
    }
  }

  @override
  String get itemCurrentImage => _itemCurrentImage;
  @override
  set itemCurrentImage(String value) {
    if (value != _itemCurrentImage) {
      _itemCurrentImage = value;
      _onItemCurrentImageChanged();
    }
  }

  @override
  String get aiResults => _aiResults;
  @override
  set aiResults(String value) {
    if (value != _aiResults) {
      _aiResults = value;
      _onAIResultsChanged();
    }
  }

  @override
  String get measureLanguage => _measureLanguage;
  @override
  set measureLanguage(String value) {
    if (value != _measureLanguage) {
      _measureLanguage = value;
      _onMeasureLanguageChanged();
    }
  }

  @override
  String get measureApplicationVersion => _measureApplicationVersion;
  @override
  set measureApplicationVersion(String value) {
    if (value != _measureApplicationVersion) {
      _measureApplicationVersion = value;
      _onMeasureApplicationVersion();
    }
  }

  @override
  DiagnosisConclusionEnum get diagnosisConclusion => _diagnosisConclusion;
  @override
  set diagnosisConclusion(DiagnosisConclusionEnum value) {
    if (value != _diagnosisConclusion) {
      _diagnosisConclusion = value;
      _onDiagnosisConclusionChanged();
    }
  }

  @override
  MeasureImageData get measureImageData => _measureImageData;
  @override
  set measureImageData(MeasureImageData value) {
    if (value != _measureImageData) {
      _measureImageData = value;
      _onMeasureImageDataChanged();
    }
  }

  @override
  List<RemedicalInfoDTO> get remedicalList => _remedicalList;
  @override
  set remedicalList(List<RemedicalInfoDTO> value) {
    if (value != _remedicalList) {
      _remedicalList = value;
      _onRemedicalListChanged();
    }
  }

  MeasureDataController(
    this.getRemedicalList,
    this.getImageInfo,
    this.getMeasureApplication,
    this.saveUserDefinedMeasureApplicationAsync,
    this.saveImage,
  );

  ///参数1：patientCode，参数2：recordCode,参数3：token
  Future<List<RemedicalItemList>> Function(String, String, String)
      getRemedicalList;

  ///参数1：remedicalCode,参数2:token
  Future<RemedicalInfoDTO?> Function(String, String) getImageInfo;

  ///参数1：测量模式参数
  Future<MeasureApplicationDTO?> Function(MeasureModeSelection)
      getMeasureApplication;

  Future<bool?> Function(Uint8List, String, String, String) saveImage;

  Future<void> Function(MeasureModeSubmit)
      saveUserDefinedMeasureApplicationAsync;

  void _onRemedicalListChanged() {
    remedicalListResultChanged.emit(this, remedicalList);
  }

  void _onAIResultsChanged() {
    aiResultsInfoChanged.emit(this, aiResults);
  }

  void _onItemCurrentImageChanged() {
    itemCurrentImageChanged.emit(this, itemCurrentImage);
  }

  void _onItemMetaListChanged() {
    itemMetaListChanged.emit(this, itemMetaList);
  }

  void _onGetMeasureApplicationChanged() {
    getMeasureApplicationChanged.emit(this, getMeasureApplicationList);
  }

  void _onDiagnosisConclusionChanged() {
    diagnosisConclusionChanged.emit(this, diagnosisConclusion);
  }

  void _onMeasureLanguageChanged() {
    measureLanguageChanged.emit(this, measureLanguage);
  }

  void _onMeasureApplicationVersion() {
    measureApplicationVersionChanged.emit(this, measureApplicationVersion);
  }

  void _onMeasureImageDataChanged() {
    measureImageDataChanged.emit(this, measureImageData);
  }
}
