import 'dart:convert';
import 'dart:typed_data';

import 'package:fis_measure/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/utils/js_utils.dart'
    if (dart.library.io) 'package:fis_measure/utils/js_utils4native.dart'
    if (dart.library.html) 'package:fis_measure/utils/js_utils.dart';
import 'package:fis_measure/view/measure/measure_tool.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

/// 颈动脉测量项页面（继承自 测量项页面）
class CarotidLeftSiderSelectMeasure extends LeftSiderSelectMeasure {
  const CarotidLeftSiderSelectMeasure({Key? key}) : super(key: key);

  @override
  FState<LeftSiderSelectMeasure> createState() =>
      _CarotidLeftSiderSelectMeasure();
}

class _CarotidLeftSiderSelectMeasure extends LeftSiderSelectMeasureState {
  late final measure3DViewController = Get.find<Measure3DViewController>();

  ///写死的颈动脉测量项
  List<String> carotidMeasureApplicationList = [
    'Distance',
    'Area',
    '%Stenosis',
    'Ant.CCA IMT',
    'Post.CCA IMT',
    'Both.CCA IMT',
  ];

  @override
  FWidget build(BuildContext context) {
    return FScrollbar(
      isAlwaysShown: true,
      child: FGridView.count(
          shrinkWrap: true,
          crossAxisCount: 2,
          childAspectRatio: 3,
          controller: ScrollController(),
          children: [
            ...carotidMeasureApplicationList.map((e) {
              if (e == activeName) {
                return _buildActivedMeasure(
                  e,
                );
              } else {
                return _buildNoActivedMeasure(e);
              }
            }).toList(),
            _buildCustomDetection('Plaque Detection'),
            _buildCustomDetection('Intima Detection')
          ]),
    );
  }

  FWidget _buildActivedMeasure(String name) {
    return FRow(
      children: [
        const FSizedBox(
          width: 12,
        ),
        FExpanded(
          child: FElevatedButton(
            onPressed: () => changeItem(name),
            child: FText(name),
            style: ElevatedButton.styleFrom(
              fixedSize: const Size.fromHeight(
                40,
              ),
            ),
          ),
        ),
        const FSizedBox(
          width: 10,
        ),
      ],
    );
  }

  FWidget _buildNoActivedMeasure(String name) {
    return FRow(
      children: [
        const FSizedBox(
          width: 12,
        ),
        FExpanded(
          child: FOutlinedButton(
            onPressed: () => changeItem(name),
            child: FText(
              name,
              style: const TextStyle(
                color: Colors.white,
              ),
            ),
            style: OutlinedButton.styleFrom(
              fixedSize: const Size.fromHeight(
                40,
              ),
              side: const BorderSide(
                color: Color.fromRGBO(124, 124, 124, 1),
              ),
              textStyle: const TextStyle(
                color: Colors.white,
              ),
            ),
          ),
        ),
        const FSizedBox(
          width: 12,
        ),
      ],
    );
  }

  FWidget _buildCustomDetection(String name) {
    return FRow(
      children: [
        const FSizedBox(
          width: 12,
        ),
        FExpanded(
          child: FOutlinedButton(
            onPressed: () => callShellDoDetection(name),
            child: FText(
              name,
              style: const TextStyle(
                color: Colors.white,
              ),
            ),
            style: OutlinedButton.styleFrom(
              fixedSize: const Size.fromHeight(
                40,
              ),
              side: const BorderSide(
                color: Color.fromRGBO(124, 124, 124, 1),
              ),
              textStyle: const TextStyle(
                color: Colors.white,
              ),
            ),
          ),
        ),
        const FSizedBox(
          width: 12,
        ),
      ],
    );
  }

  callShellDoDetection(String name) {
    if (name == 'Plaque Detection') {
      callShellMethod('getMeasureResult',
          ["{'MeasureItemType':'PlaqueDectectionMeasureItem'}"]).callMethod(
        'then',
        [
          (result) {
            if (result is String) {
              print(jsonDecode(result));
              // 正常情况会收到如下结果
              dynamic fakeResult = [
                {
                  "ExternalRectangle": "236, 133, 27, 17",
                  "IsUpward": true,
                  "PlaqueArea": 1.5841645,
                  "PlaqueEdgePoints": [
                    "236, 141", "237, 142", "238, 143", "238, 144", "239, 145",
                    "240, 146", "241, 147",
                    "242, 147", "243, 148", "244, 149", "245, 149", "246, 149",
                    "247, 149", "248, 149", "249, 149", "250, 149",
                    "251, 148", "252, 148", "253, 148", "254, 148", "255, 147",
                    "256, 147", "257, 147", "258, 147", "259, 146",
                    "260, 145", "261, 144", "261, 143", "261, 142", "261, 141",
                    "262, 140", "262, 139", "262, 138", "262, 137",
                    "262, 136", "262, 135", "262, 134", "262, 133", "261, 134",
                    "260, 135", "259, 136", "258, 137", "257, 138",
                    "256, 139", "255, 140", "254, 141", "253, 141", "252, 141",
                    "251, 141", "250, 141", "249, 141", "248, 141",
                    "247, 141", "246, 141", "245, 141", "244, 141", "243, 141",
                    "242, 141", "241, 141", "240, 141", "239, 141",
                    "238, 141", "237, 141" //
                  ],
                  "PlaqueHeight": 0.13965087,
                  "PlaquePixelHeight": 16,
                  "PlaquePixelWidth": 26,
                  "PlaqueWidth": 0.22693266
                }
              ];

              ///[Carotid] 🚧解析结果，绘制斑块
            }
          },
        ],
      );
    } else if (name == 'Intima Detection') {
      callShellMethod('getMeasureResult',
          ["{'MeasureItemType':'IntimadetectionMeasureItem'}"]).callMethod(
        'then',
        [
          (result) {
            if (result is String) {
              print(result);

              ///[Carotid] 🚧解析结果，绘制内膜

            }
          },
        ],
      );
    }
  }
}
