import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/view/cursor.dart';
import 'package:fis_theme/theme.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

part './measure_configuration_style_components.dart';

/// 初步定义的测量样式的结构体
class StyleConfiguration {
  StyleConfiguration({
    this.caliperType = MeasureCursorType.cursor01,
    this.caliperSize = 16,
    this.caliperSizeDuringMeasurement = 10,
    this.displayResultWindow = true,
    this.fontSize = '10',
    this.displayCaliperLine = false,
    this.keepMeasurementsOnPicture = true,
    this.displayDeepCaliperLine = false,
    this.showIntroductionNotes = true,
    this.measurementAutoAdjustCursor = '10',
    this.measureAutosnapRange = '10',
    this.noteFontSize = '10',
    this.displaySubItemResults = true,
    this.showSurveyNotes = true,
  });
  MeasureCursorType? caliperType;
  int? caliperSize;
  int? caliperSizeDuringMeasurement;
  bool? displayResultWindow;
  String? fontSize;
  bool? displayCaliperLine;
  bool? keepMeasurementsOnPicture;
  bool? displayDeepCaliperLine;
  bool? showIntroductionNotes;
  String? measurementAutoAdjustCursor;
  String? measureAutosnapRange;
  String? noteFontSize;
  bool? displaySubItemResults;
  bool? showSurveyNotes;

  factory StyleConfiguration.fromJson(Map<String, dynamic> map) {
    return StyleConfiguration(
      caliperType: map['caliperType'],
      caliperSize: map['caliperSize'],
      caliperSizeDuringMeasurement: map['caliperSizeDuringMeasurement'],
      displayResultWindow: map['displayResultWindow'],
      fontSize: map['fontSize'],
      displayCaliperLine: map['displayCaliperLine'],
      keepMeasurementsOnPicture: map['keepMeasurementsOnPicture'],
      displayDeepCaliperLine: map['displayDeepCaliperLine'],
      showIntroductionNotes: map['showIntroductionNotes'],
      measurementAutoAdjustCursor: map['measurementAutoAdjustCursor'],
      measureAutosnapRange: map['measureAutosnapRange'],
      noteFontSize: map['noteFontSize'],
      displaySubItemResults: map['displaySubItemResults'],
      showSurveyNotes: map['showSurveyNotes'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['caliperType'] = caliperType;
    map['caliperSize'] = caliperSize;
    map['caliperSizeDuringMeasurement'] = caliperSizeDuringMeasurement;
    map['displayResultWindow'] = displayResultWindow;
    map['fontSize'] = fontSize;
    map['displayCaliperLine'] = displayCaliperLine;
    map['keepMeasurementsOnPicture'] = keepMeasurementsOnPicture;
    map['displayDeepCaliperLine'] = displayDeepCaliperLine;
    map['showIntroductionNotes'] = showIntroductionNotes;
    map['measurementAutoAdjustCursor'] = measurementAutoAdjustCursor;
    map['measureAutosnapRange'] = measureAutosnapRange;
    map['noteFontSize'] = noteFontSize;
    map['displaySubItemResults'] = displaySubItemResults;
    map['showSurveyNotes'] = showSurveyNotes;
    return map;
  }
}

/// 下拉选择的类
class MeasureSelectModel {
  MeasureSelectModel({required this.name, required this.code});
  final String name;
  final String code;
}

class PatternDialog extends StatelessWidget {
  const PatternDialog({
    Key? key,
  }) : super(key: key);
  // final config = Get.put(MeasureToolsStyleConfiguration());

  @override
  FWidget build(BuildContext context) {
    // StyleConfiguration styleConfiguration = config.styleConfiguration;
    return FSimpleDialog(
      title: FText(
        i18nBook.measure.style.t,
        style: const TextStyle(
          color: Colors.white,
          fontSize: 18,
        ),
      ),
      isDefault: true,
      onOk: () {
        Get.back();
        // config.styleConfiguration = styleConfiguration;
      },
      onCancel: () {
        Get.back();
      },
      children: [
        FContainer(
          width: 550,
          height: 500,
          padding: const EdgeInsets.symmetric(
            vertical: 15,
            horizontal: 15,
          ),
          margin: const EdgeInsets.symmetric(
            horizontal: 15,
          ),
          child: PatternBody(styleConfiguration: StyleConfiguration()),
        ),
      ],
    );
  }
}

class MeasureTextSize {
  MeasureTextSize(this.textSize);
  final int textSize;

  @override
  String toString() {
    return '$textSize×$textSize';
  }
}

class PatternBody extends FStatefulWidget {
  const PatternBody({
    Key? key,
    required this.styleConfiguration,
  }) : super(key: key);
  final StyleConfiguration styleConfiguration;
  @override
  FState<PatternBody> createState() => _PatternBodyState();
}

class _PatternBodyState extends FState<PatternBody> {
  /// 测量项文本大小
  static final List<int> C_MEASURE_TEXT_SIZE = [
    10,
    16,
    24,
  ];

  /// 测量项标识枚举
  static final List<MeasureCursorType> C_MEASURE_CURSORTYPE = [
    MeasureCursorType.cursor01,
    MeasureCursorType.cursor02,
    MeasureCursorType.cursor03,
    MeasureCursorType.cursor04,
    MeasureCursorType.cursor05,
  ];

  /// 测量项下拉文本大小
  static final List<MeasureSelectModel> C_MEASURE_DROP_TEXT_SIZE = [
    MeasureSelectModel(
      name: '10',
      code: '10',
    ),
    MeasureSelectModel(
      name: '15',
      code: '15',
    ),
    MeasureSelectModel(
      name: '20',
      code: '20',
    ),
    MeasureSelectModel(
      name: '25',
      code: '25',
    ),
  ];
  // final controller = Get.put(MeasureHomeController());
  // final config = Get.put(MeasureToolsStyleConfiguration());
  @override
  FWidget build(BuildContext context) {
    return FListView(
      shrinkWrap: true,
      children: [
        _PatternBodyItem(
          patternBodyItemTitle: i18nBook.measure.ruler.t,
          patternBodyItem: FColumn(
            children: [
              _PatternItem(
                title: i18nBook.measure.rulerType.t,
                item: _PatternItemRadio(
                    tabList: C_MEASURE_CURSORTYPE,
                    value: widget.styleConfiguration.caliperType!,
                    isDefaultType: false,
                    onChangeState: (value) {
                      setState(() {
                        widget.styleConfiguration.caliperType = value;
                      });
                    }),
              ),
              _PatternItem(
                title: i18nBook.measure.rulerSize.t,
                item: _PatternItemRadio(
                  tabList: C_MEASURE_TEXT_SIZE,
                  value: widget.styleConfiguration.caliperSize!,
                  onChangeState: (value) {
                    setState(() {
                      widget.styleConfiguration.caliperSize = value;
                    });
                  },
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.measureRulerSize.t,
                item: _PatternItemRadio(
                  tabList: C_MEASURE_TEXT_SIZE,
                  value:
                      widget.styleConfiguration.caliperSizeDuringMeasurement!,
                  onChangeState: (value) {
                    setState(() {
                      widget.styleConfiguration.caliperSizeDuringMeasurement =
                          value;
                    });
                  },
                ),
              ),
            ],
          ),
        ),
        _PatternBodyItem(
          patternBodyItemTitle: i18nBook.measure.resultWindowConfig.t,
          patternBodyItem: FColumn(
            children: [
              _PatternItem(
                title: i18nBook.measure.showResultWindow.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.displayResultWindow = value;
                    });
                  },
                  value: widget.styleConfiguration.displayResultWindow!,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.fontSize.t,
                item: _PatternItemSelect(
                  textSize: C_MEASURE_DROP_TEXT_SIZE,
                  value: widget.styleConfiguration.fontSize!,
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.fontSize = value;
                    });
                  },
                ),
              ),
            ],
          ),
        ),
        _PatternBodyItem(
          patternBodyItemTitle: i18nBook.measure.display.t,
          isEnd: true,
          patternBodyItem: FColumn(
            children: [
              _PatternItem(
                title: i18nBook.measure.showRulerLine.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.displayCaliperLine = value;
                    });
                  },
                  value: widget.styleConfiguration.displayCaliperLine!,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.keepMeasureOnImage.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.keepMeasurementsOnPicture =
                          value;
                    });
                  },
                  value: widget.styleConfiguration.keepMeasurementsOnPicture!,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.showDepthMeasureLine.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.displayDeepCaliperLine = value;
                    });
                  },
                  value: widget.styleConfiguration.displayDeepCaliperLine!,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.showSimpleAnnotation.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.showIntroductionNotes = value;
                    });
                  },
                  value: widget.styleConfiguration.showIntroductionNotes!,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.autoAdjustCursor.t,
                item: _PatternItemSelect(
                  textSize: C_MEASURE_DROP_TEXT_SIZE,
                  value: widget.styleConfiguration.measurementAutoAdjustCursor!,
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.measurementAutoAdjustCursor =
                          value;
                    });
                  },
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.autoCaptureRange.t,
                item: _PatternItemSelect(
                  textSize: C_MEASURE_DROP_TEXT_SIZE,
                  value: widget.styleConfiguration.measureAutosnapRange!,
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.measureAutosnapRange = value;
                    });
                  },
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.annotationFontSize.t,
                item: _PatternItemSelect(
                  textSize: C_MEASURE_DROP_TEXT_SIZE,
                  value: widget.styleConfiguration.noteFontSize!,
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.noteFontSize = value;
                    });
                  },
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.showSubItemResultInMeasureWorkSheet.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.displaySubItemResults = value;
                    });
                  },
                  value: widget.styleConfiguration.displaySubItemResults!,
                ),
              ),
              _PatternItem(
                title: i18nBook.measure.showMeasureAnnotation.t,
                item: _PatternItemSwitch(
                  onChanged: (value) {
                    setState(() {
                      widget.styleConfiguration.showSurveyNotes = value;
                    });
                  },
                  value: widget.styleConfiguration.showSurveyNotes!,
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }
}
