import 'package:fis_measure/interfaces/enums/annotation.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

/// 注释项
///
// ignore: must_be_immutable
class MeasureLeftAnnotation extends StatefulWidget implements FWidget {
  const MeasureLeftAnnotation({Key? key}) : super(key: key);
  @override
  State<StatefulWidget> createState() => _MeasureLeftAnnotationState();
}

class _MeasureLeftAnnotationState extends State<MeasureLeftAnnotation> {
  // ignore: non_constant_identifier_names
  static final C_SUPPORTED_TEXTS = <String>[
    "肝左叶",
    "胆囊",
    "脾脏",
    "结石",
    "积液",
  ];

  final scrollController = ScrollController();
  final application = Get.find<IApplication>();
  late final measureHandler = Get.find<MeasureHandler>();

  @override
  void initState() {
    Future.delayed(const Duration(milliseconds: 1000), () {
      application.switchAnnotation(AnnotationType.label, C_SUPPORTED_TEXTS[0]);
    });
    // application.switchAnnotation(AnnotationType.arrow);
    // application.switchAnnotation(AnnotationType.input);
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: 300,
      child: Scrollbar(
        controller: scrollController,
        isAlwaysShown: true,
        child: GridView.count(
          controller: scrollController,
          crossAxisCount: 2,
          childAspectRatio: 3,
          children: C_SUPPORTED_TEXTS.map((e) {
            return _buildNoteTools(e);
          }).toList(),
        ),
      ),
    );
  }

  Widget _buildNoteTools(String tools) {
    const style = TextStyle(color: Colors.white, fontSize: 16);
    const dragStyle = TextStyle(color: Colors.amber, fontSize: 18);
    return Row(
      mainAxisSize: MainAxisSize.max,
      children: [
        const FSizedBox(
          width: 12,
        ),
        Expanded(
          child: Draggable<String>(
            data: tools,
            dragAnchorStrategy: (data, context, offset) {
              return Offset.zero;
            },
            child: OutlinedButton(
              child: Text(tools, style: style),
              onPressed: () {
                measureHandler.changedAnnotationType = AnnotationType.label;
                application.switchAnnotation(AnnotationType.label, tools);
              },
              style: OutlinedButton.styleFrom(
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(4),
                ),
                side: const BorderSide(
                  color: Color.fromRGBO(124, 124, 124, 1),
                ),
                fixedSize: const Size.fromHeight(40),
              ),
            ),
            feedback: Material(
              color: Colors.transparent,
              child: Text(tools, style: dragStyle),
            ),
            onDragStarted: () {
              measureHandler.changedAnnotationType = AnnotationType.label;
              application.switchAnnotation(AnnotationType.label, tools);
            },
          ),
        ),
        const FSizedBox(
          width: 12,
        ),
      ],
    );
  }
}
