import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

/// 测量项页面
class LeftSiderSelectMeasure extends FStatefulWidget {
  const LeftSiderSelectMeasure({Key? key}) : super(key: key);

  @override
  FState<LeftSiderSelectMeasure> createState() => LeftSiderSelectMeasureState();
}

class LeftSiderSelectMeasureState extends FState<LeftSiderSelectMeasure> {
  final ScrollController scrollController = ScrollController();

  late final application = Get.find<IApplication>();
  late final measureHandler = Get.find<MeasureHandler>();

  /// 数据
  late final measureData = Get.find<MeasureDataController>();

  String activeName = '';
  String activeMode = '';

  void changeItem(String? name) {
    activeName = name ?? '';
    application.switchItemByName(activeName);
    if (mounted) {
      setState(() {});
    }
  }

  void itemMetaList(Object s, dynamic e) {
    try {
      if (e != null) {
        setState(() {});
      }
    } catch (e) {
      debugPrint("callmethod fail " + e.toString());
    }
  }

  void getMeasureApplicationChanged(sender, e) {
    setState(() {});
  }

  void visualsLoaded(sender, e) async {
    final applicationModes = application.visuals[0].modes;
    activeMode = applicationModes[0].modeType.toString().split('.')[1];
    var measureModeSelection = MeasureModeSelection(
      application.applicationName,
      application.categoryName,
      application.isThirdPart ? ['TPPTissue'] : [activeMode],
    );
    measureHandler.measureModeChanged = measureModeSelection;
    var measureApplicationDTO =
        await measureData.getMeasureApplication.call(measureModeSelection);
    if (measureApplicationDTO != null) {
      /// 模式版本
      measureData.measureApplicationVersion =
          measureApplicationDTO.version ?? '';

      ///模式列表
      var models = measureApplicationDTO.availableModes;
      if (models != null && models.isNotEmpty) {
        ///群组列表
        var groups = models[0].availableGroups;
        if (groups != null && groups.isNotEmpty) {
          ///项目列表
          var folders = groups[0].availableFolders;
          if (folders != null && folders.isNotEmpty) {
            measureData.getMeasureApplicationList =
                folders[0].workingItemNames ?? [];
            if (folders[0].availableItems != null) {
              measureData.itemMetaList = folders[0].availableItems!.toList();
            }
            print('测量方法集: ${measureData.getMeasureApplicationList.toString()}');
            if (measureData.getMeasureApplicationList.isNotEmpty) {
              changeItem(measureData.getMeasureApplicationList[0]);
            }
          }
        }
      }
    }
    setState(() {});
  }

  @override
  void initState() {
    application.visualsLoaded.addListener(visualsLoaded);

    measureData.itemMetaListChanged.addListener(itemMetaList);
    measureData.getMeasureApplicationChanged
        .addListener(getMeasureApplicationChanged);
    super.initState();
  }

  @override
  dispose() {
    super.dispose();
    measureData.itemMetaListChanged.removeListener(itemMetaList);
    measureData.getMeasureApplicationChanged
        .removeListener(getMeasureApplicationChanged);
    application.visualsLoaded.removeListener(visualsLoaded);
  }

  @override
  FWidget build(BuildContext context) {
    return FScrollbar(
      isAlwaysShown: true,
      child: FGridView.count(
        shrinkWrap: true,
        crossAxisCount: 2,
        childAspectRatio: 3,
        controller: ScrollController(),
        children: measureData.getMeasureApplicationList.map((e) {
          if (e == activeName) {
            return _buildActivedMeasure(
              e,
            );
          } else {
            return _buildNoActivedMeasure(e);
          }
        }).toList(),
      ),
    );
  }

  FWidget _buildActivedMeasure(String name) {
    return FRow(
      children: [
        const FSizedBox(
          width: 12,
        ),
        FExpanded(
          child: FElevatedButton(
            onPressed: () => changeItem(name),
            child: FText(name),
            style: ElevatedButton.styleFrom(
              fixedSize: const Size.fromHeight(
                40,
              ),
            ),
          ),
        ),
        const FSizedBox(
          width: 10,
        ),
      ],
    );
  }

  FWidget _buildNoActivedMeasure(String name) {
    return FRow(
      children: [
        const FSizedBox(
          width: 12,
        ),
        FExpanded(
          child: FOutlinedButton(
            onPressed: () => changeItem(name),
            child: FText(
              name,
              style: const TextStyle(
                color: Colors.white,
              ),
            ),
            style: OutlinedButton.styleFrom(
              fixedSize: const Size.fromHeight(
                40,
              ),
              side: const BorderSide(
                color: Color.fromRGBO(124, 124, 124, 1),
              ),
              textStyle: const TextStyle(
                color: Colors.white,
              ),
            ),
          ),
        ),
        const FSizedBox(
          width: 12,
        ),
      ],
    );
  }
}
