import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/enums/annotation.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/interfaces/process/workspace/exam_info.dart';
import 'package:fis_measure/interfaces/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/process/layout/configuration.dart';
import 'package:fis_measure/process/workspace/measure_controller.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/view/3d_view/measure_3d_view_switcher.dart';
import 'package:fis_measure/view/measure/measure_images_bar.dart';
import 'package:fis_measure/view/measure/measure_left_annotation.dart';
import 'package:fis_measure/view/measure/measure_player.dart';
import 'package:fis_measure/view/measure/measure_tool.dart';
import 'package:fis_measure/view/measure/carotid_measure_tool.dart';
import 'package:fis_measure/view/measure/tool_chest_title.dart';
import 'package:fis_measure/view/player/control_board/operate_bar.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

/// 测量主页面
class MeasureMainPage extends StatefulWidget implements FWidget {
  const MeasureMainPage(
      this.token, this.recordCode, this.patientCode, this.remedicalCode,
      {Key? key})
      : super(key: key);

  final String token;
  final String patientCode;
  final String remedicalCode;
  final String recordCode;

  @override
  State<StatefulWidget> createState() => _MeasureMainPageState();
}

class _MeasureMainPageState extends State<MeasureMainPage> {
  /// 数据
  final measureData = Get.find<MeasureDataController>();

  /// webview 控制器
  final measure3DViewController = Get.find<Measure3DViewController>();
  late final application = Get.find<IApplication>();

  /// 页面loadding
  bool loaded = false;

  // /// 图片loadding
  late bool imageLoaded = false;

  ///检查图片信息表
  List<ExamImageInfo> examImageInfoList = [];

  late final measureHandler = Get.find<MeasureHandler>();

  /// 测量控制器
  late MeasureController measureController = Get.put(MeasureController(
    "",
    imagesFetchFunc: (code) async {
      return examImageInfoList;
    },
  ));

  /// 获取测量图片所需的图片组 并且写入控制器中 加载
  void getExamImageInfoList(List<RemedicalInfoDTO> remedicals) async {
    for (var element in remedicals) {
      examImageInfoList.add(
        ExamImageInfo(
          element.terminalImages!.imageUrl!,
          element.terminalImages!.previewUrl!,
        ),
      );
    }

    measureController = Get.put(MeasureController(
      "",
      imagesFetchFunc: (code) async {
        return examImageInfoList;
      },
    ));

    await measureController.load();
    int selectedImageIndex = -1;
    if (examImageInfoList
        .any((element) => element.url == measureData.itemCurrentImage)) {
      ExamImageInfo selectedImage = examImageInfoList.firstWhere(
        (element) => element.url == measureData.itemCurrentImage,
      );
      selectedImageIndex = examImageInfoList.indexOf(selectedImage);
      measureController.examInfo.selectedImageIndex = selectedImageIndex;
    }
  }

  void changeImage(sender, e) {
    imageLoaded = e;
    setState(() {});
  }

  void changeFullScreenState(sender, e) {
    setState(() {});
  }

  @override
  void initState() {
    loadLayoutConfig();
    getImageInfo();
    measureHandler.onChangeImageLoaded.addListener(changeImage);
    measureHandler.onChangeFullScreenState.addListener(changeFullScreenState);
    super.initState();
  }

  void getImageInfo() {
    measureData.measureImageData = MeasureImageData(
      patientCode: widget.patientCode,
      recordCode: widget.recordCode,
      remedicalCode: widget.remedicalCode,
    );
  }

  /// 加载图像布局配置
  void loadLayoutConfig() async {
    await LayoutConfiguration.ins.loadData();
    setState(() {
      // 加载图像数据
      _initData();
    });
  }

  void onImageLoaded(Object sender, ExamImageInfo? e) async {
    measureHandler.changeImageLoaded = true;
    if (!mounted) return;
    final currentImage = measureData.remedicalList.firstWhereOrNull(
        (element) => element.terminalImages!.imageUrl == e!.url);
    if (currentImage != null) {
      /// 获取图片详细信息
      var remedicalInfo = await measureData.getImageInfo(
        currentImage.remedicalCode ?? '',
        widget.token,
      );
      if (remedicalInfo != null) {
        measureData.aiResults = remedicalInfo.diagnosisResult ?? '';
        measure3DViewController.initParams();
        measure3DViewController.curMeasureMode = MeasureMode.vidMode;

        /// ai 良恶性 判断是否有ai
        measureData.diagnosisConclusion = remedicalInfo.diagnosisConclusion;
        if (remedicalInfo.carotidResult != null) {
          /// [Carotid] ✅详情传入测量信息
          measure3DViewController.carotidResult = remedicalInfo.carotidResult!;

          /// [Carotid] ✅需要在此通知 controller 存在颈动脉信息
          measure3DViewController.exist3DData = true;
        } else {
          measure3DViewController.exist3DData = false;
        }

        try {
          if (e != null) {
            Future.delayed(const Duration(milliseconds: 100), () {
              measureController.playerController.play();
            });
            setState(() {
              loaded = true;
              measureHandler.changeImageLoaded = false;
            });
          }
        } catch (error) {
          setState(() {
            loaded = true;
            measureHandler.changeImageLoaded = false;
          });
        }
      }
    }
  }

  @override
  void dispose() {
    measureHandler.onChangeImageLoaded.removeListener(changeImage);
    measureHandler.onChangeFullScreenState
        .removeListener(changeFullScreenState);
    super.dispose();
  }

  @override
  FWidget build(BuildContext context) {
    FWidget body;
    if (!loaded) {
      const loadingWidget = FCenter(child: FCircularProgressIndicator());
      body = FRow(
        children: const [
          FExpanded(
            child: loadingWidget,
          ),
        ],
      );
    } else {
      body = FRow(
        children: [
          if (!measureHandler.fullScreenState) _MeasureLeftBoard(),
          const FVerticalDivider(),
          FExpanded(
            child: FColumn(
              mainAxisSize: MainAxisSize.max,
              children: [
                FExpanded(
                    child: FStack(
                  children: [
                    if (!imageLoaded)
                      MeasureRightBoard(
                        key: ValueKey(measureData.itemCurrentImage),
                      ),
                    if (imageLoaded)
                      const FCenter(
                        child: FCircularProgressIndicator(),
                      ),
                  ],
                )),
                if (!measureHandler.fullScreenState) const MeasureImagesBar(),
              ],
            ),
          ),
        ],
      );
    }

    return FCenter(
      child: FContainer(
        color: const Color.fromRGBO(70, 70, 70, 1),
        child: body,
      ),
    );
  }

  void _initData() async {
    List<RemedicalInfoDTO> remedicals = [];
    List<Ultra3DResourceInfo> ultra3DResourceInfos = [];
    loaded = false;

    var value = await measureData.getRemedicalList
        .call(widget.patientCode, widget.recordCode, widget.token);
    for (var remedicalItemList in value) {
      remedicals.addAll(remedicalItemList.remedicalList ?? []);
    }

    /// [Carotid] ✅遍历出颈动脉信息列表,传给壳子
    for (var remedical in remedicals) {
      if (remedical.carotidResult != null) {
        ultra3DResourceInfos.add(Ultra3DResourceInfo(remedical.carotidResult!));
      }
    }
    measure3DViewController.ultra3DResourceInfoList = ultra3DResourceInfos;
    measure3DViewController.recordId = widget.recordCode;
    measure3DViewController.notifyShellLoadAllModel();
    measureData.remedicalList = remedicals;
    var remedicalInfo =
        await measureData.getImageInfo.call(widget.remedicalCode, widget.token);
    if (remedicalInfo != null) {
      measureData.aiResults = remedicalInfo.diagnosisResult ?? '';

      if (remedicalInfo.terminalImages != null) {
        loaded = true;
        measureData.itemCurrentImage =
            remedicalInfo.terminalImages!.imageUrl ?? '';
        getExamImageInfoList(remedicals);
      }
    }

    measureController.imageLoaded.removeListener(onImageLoaded);
    measureController.imageLoaded.addListener(onImageLoaded);
  }
}

/// 测量左边操作页面
class _MeasureLeftBoard extends StatefulWidget implements FWidget {
  @override
  State<_MeasureLeftBoard> createState() => _MeasureLeftBoardState();
}

class _MeasureLeftBoardState extends State<_MeasureLeftBoard> {
  final measureHandler = Get.find<MeasureHandler>();
  late final application = Get.find<IApplication>();
  final playerController = Get.find<IPlayerController>();
  final measure3DViewController = Get.find<Measure3DViewController>();
  bool get exist3DData => measure3DViewController.exist3DData;
  bool get isShell => measure3DViewController.isShell;

  /// 是否显示颈动脉2D指定的测量项
  bool showCarotid2DSelectMeasure = false;

  bool get isMeasureTool => measureHandler.changedTab == TagEnum.MeasureTool;
  bool get isArrowMeasureAnnotationType =>
      measureHandler.changedAnnotationType == AnnotationType.arrow;
  void onChangedTab(
    Object sender,
    TagEnum? e,
  ) {
    setState(() {});
  }

  @override
  void initState() {
    measureHandler.onChangedTab.addListener(onChangedTab);
    measure3DViewController.updatePlayerMode.addListener(_onModeChanged);
    super.initState();
  }

  @override
  void dispose() {
    measureHandler.onChangedTab.removeListener(onChangedTab);
    measure3DViewController.updatePlayerMode.removeListener(_onModeChanged);
    super.dispose();
  }

  /// 模式改变触发更新
  void _onModeChanged(Object s, MeasureMode mode) {
    print("Measure main page 测量模式更新: ${mode.toString()}");
    switch (mode) {
      case MeasureMode.vidMode:
        setState(() {
          showCarotid2DSelectMeasure = false;
        });
        break;
      case MeasureMode.carotid2DMode:
        setState(() {
          showCarotid2DSelectMeasure = true;
        });
        break;
      case MeasureMode.carotid3DMode:
        setState(() {
          showCarotid2DSelectMeasure = false;
        });
        break;
    }
  }

  @override
  FWidget build(BuildContext context) {
    return FColumn(
      mainAxisSize: MainAxisSize.max,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        const LeftSiderHold(),
        if (!isMeasureTool) ...[
          FExpanded(
            child: FContainer(
              width: 300,
              key: UniqueKey(),
              child: const MeasureLeftAnnotation(),
            ),
          ),
          _MeasureArrow(),
        ] else ...[
          FExpanded(
            child: FContainer(
              width: 300,
              key: UniqueKey(),
              child: showCarotid2DSelectMeasure
                  ? const CarotidLeftSiderSelectMeasure()
                  : const LeftSiderSelectMeasure(),
            ),
          ),
          isShell && exist3DData
              ? Measure3DViewSwitcher(
                  measure3DViewController,
                )
              : FContainer(),
          const OperateBar(),
        ]
      ],
    );
  }
}

class _MeasureArrow extends FStatefulWidget {
  @override
  FState<_MeasureArrow> createState() => _MeasureArrowState();
}

class _MeasureArrowState extends FState<_MeasureArrow> {
  final measureHandler = Get.find<MeasureHandler>();

  final application = Get.find<IApplication>();
  bool get isArrowMeasureAnnotationType =>
      measureHandler.changedAnnotationType == AnnotationType.arrow;

  void onChangedAnnotationType(
    Object sender,
    AnnotationType? e,
  ) {
    setState(() {});
  }

  @override
  void initState() {
    measureHandler.onChangedAnnotationType.addListener(onChangedAnnotationType);
    super.initState();
  }

  @override
  void dispose() {
    measureHandler.onChangedAnnotationType
        .removeListener(onChangedAnnotationType);
    super.dispose();
  }

  @override
  FWidget build(BuildContext context) {
    return FContainer(
      height: 100,
      width: 300,
      child: FCenter(
        child: FColumn(
          children: [
            FInk(
              child: FInkWell(
                onTap: () {
                  measureHandler.changedAnnotationType = AnnotationType.arrow;

                  application.switchAnnotation(AnnotationType.arrow);
                },
                child: FIcon(
                  Icons.call_made_rounded,
                  color:
                      isArrowMeasureAnnotationType ? Colors.blue : Colors.white,
                  size: 30,
                ),
              ),
            ),

            /// TODO 翻译
            FText(
              '箭头',
              style: isArrowMeasureAnnotationType
                  ? const TextStyle(
                      color: Colors.blue,
                      fontSize: 14,
                    )
                  : const TextStyle(
                      color: Colors.white,
                      fontSize: 14,
                    ),
            ),
          ],
        ),
      ),
    );
  }
}
