import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/interfaces/process/modes/mode.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/view/measure/measure_config/measure_configuation_page.dart';
import 'package:fis_measure/view/measure/measure_config/widgets/measure_configuration_style.dart';
import 'package:fis_theme/theme.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

/// 下拉选择的类
class MeasureSelectModel {
  MeasureSelectModel({required this.name, required this.code});
  final String name;
  final String code;
}

enum TagEnum {
  MeasureTool,
  NodesTool,
}

/// 工具箱 样式配置之类
class LeftSiderHold extends StatelessWidget implements FWidget {
  const LeftSiderHold({Key? key}) : super(key: key);

  @override
  FWidget build(BuildContext context) {
    return FContainer(
      width: 300,
      padding: const EdgeInsets.symmetric(
        horizontal: 10,
        vertical: 15,
      ),
      child: FColumn(
        key: UniqueKey(),
        mainAxisSize: MainAxisSize.max,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          FRow(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              // 工具箱文案
              FText(
                i18nBook.measure.toolbox.t,
                style: const TextStyle(
                  color: Colors.white,
                ),
              ),
              _LeftSiderHoldAllStyleConfig(),
            ],
          ),
          _LeftSiderTabBar(),
          const _LeftMeasureTools(),
          _LeftSelectInput(),
          _LeftPulldown(),
        ],
      ),
    );
  }
}

/// 测量样式 和配置
class _LeftSiderHoldAllStyleConfig extends StatelessWidget implements FWidget {
  @override
  FWidget build(BuildContext context) {
    return FRow(
      children: [
        FTextButton(
          onPressed: () {
            Get.dialog(
              const PatternDialog(),
            );
          },
          child: FText(
            i18nBook.measure.style.t,
            style: TextStyle(
              color: FTheme.ins.colorScheme.primary,
            ),
          ),
        ),
        FTextButton(
          onPressed: () {
            Get.dialog(
              const MeasureConfigurationPage(),
            );
          },
          child: FText(
            i18nBook.measure.config.t,
            style: TextStyle(
              color: FTheme.ins.colorScheme.primary,
            ),
          ),
        ),
      ],
    );
  }
}

/// 测量工具 注释工具Tab
class _LeftSiderTabBar extends StatefulWidget implements FWidget {
  @override
  State<_LeftSiderTabBar> createState() => _LeftSiderTabBarState();
}

class _LeftSiderTabBarState extends State<_LeftSiderTabBar> {
  final measureHandler = Get.find<MeasureHandler>();

  @override
  FWidget build(BuildContext context) {
    return FRow(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        _buildTab(
          TagEnum.MeasureTool,
          i18nBook.measure.measureTool.t,
        ),
        _buildTab(
          TagEnum.NodesTool,
          i18nBook.measure.annotationTool.t,
        ),
      ],
    );
  }

  FWidget _buildTab(TagEnum tag, String title) {
    return FTextButton(
      style: ButtonStyle(
        padding: MaterialStateProperty.all(
          const EdgeInsets.all(
            6,
          ),
        ),
      ),
      onPressed: () {
        measureHandler.changedTab = tag;
        setState(() {});
      },
      child: FText(
        title,
        style: TextStyle(
          color: measureHandler.changedTab == tag
              ? FTheme.ins.colorScheme.primary
              : Colors.white,
        ),
      ),
    );
  }
}

/// 测量方法 Har
class _LeftMeasureTools extends StatefulWidget implements FWidget {
  const _LeftMeasureTools();

  @override
  State<_LeftMeasureTools> createState() => _LeftMeasureToolsState();
}

class _LeftMeasureToolsState extends State<_LeftMeasureTools> {
  late final application = Get.find<IApplication>();
  late final measureHandler = Get.find<MeasureHandler>();

  /// 数据
  late final measureData = Get.find<MeasureDataController>();
  late List<IMode> applicationModes = [];

  String activeName = '';

  /// 初始化名字
  String activeMode = '';

  void changeItem(String? name) {
    activeName = name ?? '';
    application.switchItemByName(activeName);
    if (mounted) {
      setState(() {});
    }
  }

  void visualsLoaded(sender, e) {
    applicationModes = application.visuals[0].modes;
    activeMode = applicationModes[0].modeType.toString().split('.')[1];
    setState(() {});
  }

  Future<void> getName(String modeType) async {
    activeMode = modeType;
    var measureModeSelection = MeasureModeSelection(
      application.applicationName,
      application.categoryName,
      application.isThirdPart ? ['TPPTissue'] : [modeType],
    );
    measureHandler.measureModeChanged = measureModeSelection;
    var measureApplicationDTO =
        await measureData.getMeasureApplication.call(measureModeSelection);
    if (measureApplicationDTO != null) {
      /// 模式版本
      measureData.measureApplicationVersion =
          measureApplicationDTO.version ?? '';

      ///模式列表
      var models = measureApplicationDTO.availableModes;
      if (models != null && models.isNotEmpty) {
        ///群组列表
        var groups = models[0].availableGroups;
        if (groups != null && groups.isNotEmpty) {
          ///项目列表
          var folders = groups[0].availableFolders;
          if (folders != null && folders.isNotEmpty) {
            measureData.getMeasureApplicationList =
                folders[0].workingItemNames ?? [];
            if (folders[0].availableItems != null) {
              measureData.itemMetaList = folders[0].availableItems!.toList();
            }
            if (measureData.getMeasureApplicationList.isNotEmpty) {
              changeItem(measureData.getMeasureApplicationList[0]);
            }
          }
        }
      }
    }
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    application.visualsLoaded.addListener(visualsLoaded);
    application.visualAreaChanged.addListener((sender, e) {
      print(e.mode.modeType.toString().split('.')[1]);
      getName(e.mode.modeType.toString().split('.')[1]);
      // application.switchMode(e.mode.modeType.toString().split('.')[1]);
    });
  }

  @override
  void dispose() {
    super.dispose();
    application.visualsLoaded.removeListener(visualsLoaded);
  }

  @override
  FWidget build(BuildContext context) {
    return FContainer(
      width: 300,
      height: 50,
      alignment: Alignment.topCenter,
      padding: const EdgeInsets.symmetric(vertical: 8),
      child: FGridView.count(
        shrinkWrap: true,
        crossAxisCount: 4,
        crossAxisSpacing: 10,
        mainAxisSpacing: 10,
        childAspectRatio: 1 / 0.5,
        children: applicationModes.map((e) {
          final bool isACtiveModes =
              activeMode == e.modeType.toString().split('.')[1];

          return FInkWell(
            onTap: () {
              getName(e.modeType.toString().split('.')[1]);
            },
            child: FContainer(
              alignment: Alignment.center,
              decoration: BoxDecoration(
                border: isACtiveModes
                    ? Border.all(
                        color: Colors.blue,
                      )
                    : Border.all(
                        color: Colors.grey,
                      ),
                borderRadius: BorderRadius.circular(4),
                color: isACtiveModes ? Colors.blue : Colors.grey,
              ),
              child: FText(
                e.name,
                style: const TextStyle(
                  color: Colors.white,
                ),
              ),
            ),
          );
        }).toList(),
      ),
    );
  }
}

class _LeftSelectInput extends StatelessWidget implements FWidget {
  @override
  FWidget build(BuildContext context) {
    return FContainer(
      child: FRow(
        children: [
          FExpanded(
            child: FContainer(
              child: FRow(
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  FBorderInput(
                    hintSize: 16,
                    contentSize: 16,
                    maxLength: 20,
                    borderColor: const Color.fromARGB(255, 187, 180, 180),
                    suffixIcon: FMaterial(
                      color: Colors.transparent,
                      child: FIconButton(
                        onPressed: () {},
                        icon: const FIcon(
                          Icons.search,
                        ),
                      ),
                    ),
                    hintText: i18nBook.common.search.t,
                    onChanged: (value) {},
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class _LeftPulldown extends StatelessWidget implements FWidget {
  @override
  FWidget build(BuildContext context) {
    return FContainer(
      margin: const EdgeInsets.symmetric(vertical: 4),
      child: FSelect<MeasureSelectModel, String>(
        source: [
          MeasureSelectModel(
            name: 'General',
            code: 'General',
          )
        ],
        hintText: i18nBook.measure.selectMeasureItem.t,
        value: 'General',
        clearable: false,
        height: 36,
        width: 400,
        optionLabelExtractor: (data) => data.name,
        optionValueExtractor: (data) => data.code,
        onSelectChanged: (value, index) {},
      ),
    );
  }
}
