import 'dart:convert';

import 'package:fis_measure/view/paint/ai_patint_controller.dart';
import 'package:fis_measure/view/paint/date_structure.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

/// AI诊断结果
class ResultInfo extends StatelessWidget {
  final List<AIDetectedObject> aiDetectedObject;
  // final application = Get.find<IApplication>();
  late final aiPatintController = Get.find<AiPatintController>();

  late AIDetectedObject aiDetectedObjectItem;

  ResultInfo(this.aiDetectedObject, {Key? key}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    final description =
        aiDetectedObject[aiPatintController.state.aiResultIndex].descriptions;
    final descriptions = description!.length > 1
        ? jsonDecode(description[description.length - 1].value ?? '')
        : '';

    return Container(
      decoration: BoxDecoration(
        border: Border.all(
          color: Colors.grey,
        ),
        borderRadius: BorderRadius.circular(4),
        color: Colors.transparent,
      ),
      padding: const EdgeInsets.only(bottom: 10),
      child: Column(
        children: [
          Row(
            mainAxisSize: MainAxisSize.max,
            children: [
              Expanded(
                child: Container(
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(4),
                      topRight: Radius.circular(4),
                    ),
                    color: Color.fromRGBO(54, 169, 206, 1),
                  ),
                  padding: const EdgeInsets.only(
                    top: 4,
                    bottom: 10,
                    left: 8,
                    right: 8,
                  ),
                  child: const Text(
                    'AI诊断结果',
                    style: TextStyle(
                      color: Colors.white,
                    ),
                  ),
                ),
              )
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Container(
                padding: const EdgeInsets.only(
                  left: 10,
                ),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: [
                    _buildTitle(
                      '疾病标签',
                      _buildAITitle(),
                    ),
                    if (descriptions != '')
                      _buildTitle(
                        '病灶大小',
                        _buildLesionSize(
                          descriptions['HorizontalLengthInPixel'],
                          descriptions['VerticalLengthInPixel'],
                        ),
                      )
                  ],
                ),
              ),
              Container(
                margin: const EdgeInsets.symmetric(
                  vertical: 15,
                ),
                child: SizedBox(
                  width: 70,
                  height: 70,
                  child: Stack(
                    children: [
                      SizedBox(
                        width: 70,
                        height: 70,
                        child: Obx(() {
                          final aiDetected = aiDetectedObject[
                              aiPatintController.state.aiResultIndex];
                          return CircularProgressIndicator(
                            valueColor: AlwaysStoppedAnimation(
                              _buildAITextColor(
                                aiDetected.label,
                              ),
                            ),
                            backgroundColor: Colors.grey,
                            value: aiDetected.confidence,
                          );
                        }),
                      ),
                      Center(
                        child: SizedBox(
                          width: 45,
                          height: 45,
                          child: Obx(() {
                            final confidence = aiDetectedObject[
                                    aiPatintController.state.aiResultIndex]
                                .confidence;
                            return Text(
                              '可能性${(confidence * 100).toStringAsFixed(1)}%',
                              style: const TextStyle(
                                color: Colors.white,
                              ),
                            );
                          }),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }

  Widget _buildLesionSize(
      int horizontalLengthInPixel, int verticalLengthInPixel) {
    return Text(
      (horizontalLengthInPixel * aiPatintController.unitsPhysicalPixels)
              .toStringAsFixed(2)
              .toString() +
          'cm x' +
          (verticalLengthInPixel * aiPatintController.unitsPhysicalPixels)
              .toStringAsFixed(2)
              .toString() +
          'cm',
      style: const TextStyle(color: Colors.white),
    );
  }

  Widget _buildTitle(String label, Widget value) {
    return Column(
      mainAxisSize: MainAxisSize.max,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Text(
          label,
          style: const TextStyle(
            color: Color.fromRGBO(54, 169, 206, 1),
          ),
        ),
        value,
        const SizedBox(
          height: 5,
        ),
      ],
    );
  }

  Color _buildAITextColor(int label) {
    switch (label) {
      case 0:
        return Colors.lightBlue;
      case 1:
      case 2:
      case 3:
        return Colors.greenAccent;
      default:
        return Colors.redAccent;
    }
  }

  Widget _buildAITitle() {
    switch (aiPatintController.diagnosisOrgan) {
      case DiagnosisOrganEnum.Breast:
        return Obx(() {
          aiDetectedObjectItem =
              aiDetectedObject[aiPatintController.state.aiResultIndex];
          return _buildBreastDescription(aiDetectedObjectItem.label);
        });

      case DiagnosisOrganEnum.Liver:
        return Obx(() {
          aiDetectedObjectItem =
              aiDetectedObject[aiPatintController.state.aiResultIndex];
          return _buildLiverDescription(aiDetectedObjectItem.label);
        });
      default:
        return const SizedBox();
    }
  }

  Widget _buildBreastDescription(int label) {
    switch (label) {
      case 0:
        return _buildDescription('未见明显异常');
      case 1:
        return _buildDescription('脂肪瘤');
      case 2:
        return _buildDescription('BI-RADS 2');
      case 3:
        return _buildDescription('BI-RADS 3');
      case 4:
        return _buildDescription('BI-RADS 4a');
      case 5:
        return _buildDescription('BI-RADS 4b');
      case 6:
        return _buildDescription('BI-RADS 4c');
      case 7:
        return _buildDescription('BI-RADS 5');
      case 8:
        return _buildDescription('未见明显异常');
      default:
        return _buildDescription(null);
    }
  }

  /// TODO 翻译
  Widget _buildLiverDescription(int label) {
    switch (label) {
      case 0:
        return _buildDescription('未见明显异常');
      case 1:
        return _buildDescription('肝内强回声灶');
      case 2:
        return _buildDescription('肝血管瘤');
      case 3:
        return _buildDescription('肝囊肿');
      case 4:
        return _buildDescription('肝癌可能');
      case 5:
        return _buildDescription('脂肪肝');
      case 6:
        return _buildDescription('肝脏弥漫性病变声像图改变');
      case 7:
        return _buildDescription('肝硬化');
      case 8:
        return _buildDescription('多囊肝');
      default:
        return _buildDescription(null);
    }
  }

  Widget _buildDescription(
    String? title,
  ) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        if (title != null) ...[
          const SizedBox(
            height: 5,
          ),
          Text(
            title,
            style: const TextStyle(color: Colors.white),
          ),
        ],
      ],
    );
  }
}
