import 'dart:ui' as ui;
import 'package:fis_measure/interfaces/process/calculators/output.dart';
import 'package:fis_measure/interfaces/process/modes/mode.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/process/items/item_feature.dart';
import 'package:fis_measure/values/colors.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class MeasureResultPanel extends StatefulWidget {
  const MeasureResultPanel({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _MeasureResultPanelState();
}

class _MeasureResultPanelState extends State<MeasureResultPanel> {
  final _scrollController = ScrollController();
  final application = Get.find<IApplication>();

  final List<OutputItem> outputs = [];
  final _outputMap = <IMode, List<OutputItem>>{};
  int idStrLength = 1;

  bool get hasOutputs => outputs.isNotEmpty;

  @override
  Widget build(BuildContext context) {
    if (!hasOutputs) return const SizedBox();

    final child = Container(
      constraints: const BoxConstraints(
        maxWidth: 200,
        minWidth: 150,
        maxHeight: 400,
      ),
      decoration: BoxDecoration(
        border: Border.all(
          color: MeasureColors.ResultBorder,
          width: 2,
        ),
        borderRadius: BorderRadius.circular(4),
      ),
      // padding: const EdgeInsets.only(left: 2),
      child: FThemeWidget(
        data: Theme.of(context).copyWith(
          scrollbarTheme: ScrollbarThemeData(
            thumbColor: MaterialStateProperty.all(
              MeasureColors.ResultScrollbar,
            ),
            crossAxisMargin: 1,
          ),
        ),
        child: FScrollbar(
          controller: _scrollController,
          isAlwaysShown: true,
          radius: ui.Radius.zero,
          child: FSingleChildScrollView(
            controller: _scrollController,
            child: FRepaintBoundary(
              child: FIntrinsicHeight(
                child: FIntrinsicWidth(
                  child: FContainer(
                    margin: const EdgeInsets.all(8),
                    child: FColumn(
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: _buildMainList(),
                    ),
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
    return child;
    return Draggable(
      child: child,
      feedback: Material(
        color: Colors.transparent,
        child: child,
      ),
      childWhenDragging: const SizedBox(),
      onDragEnd: (details) {
        final renderBox = context.findRenderObject()! as RenderBox;
        print(context.mediaQuerySize);
        // Application.resultPanelOffset = renderBox.globalToLocal(details.offset);
      },
      onDragUpdate: (details) {
        // final renderBox = context.findRenderObject()! as RenderBox;
      },
    );
  }

  @override
  void initState() {
    _updateOutputs();
    application.updateRenderReady.addListener(_onMeasureRerenderReady);
    super.initState();
  }

  @override
  void dispose() {
    application.updateRenderReady.removeListener(_onMeasureRerenderReady);
    super.dispose();
  }

  List<FWidget> _buildMainList() {
    final list = <FWidget>[];
    list.addAll(_buildModeOutput(application.applicationName, outputs));
    return list;
  }

  List<FWidget> _buildModeOutput(
      String title, List<OutputItem<dynamic>> dataList) {
    final list = <FWidget>[
      const FSizedBox(height: 6),
      _buildTitle(title),
      _buildTitleDivider(),
    ];

    final length = dataList.length;
    for (var i = 0; i < length; i++) {
      final output = dataList[i];
      final idStr = _getIdDisplayStr(output.id);
      list.add(const FSizedBox(height: 4));
      list.add(_buildLine('$idStr ${output.description}'));
    }

    return list;
  }

  FWidget _buildLine(String text) {
    return FText(
      text,
      style: const TextStyle(
        fontSize: 16,
        color: MeasureColors.Primary,
      ),
    );
  }

  FWidget _buildTitle(String title) {
    return FText(
      title,
      style: const TextStyle(
        fontSize: 14,
        color: MeasureColors.Primary,
      ),
    );
  }

  FWidget _buildTitleDivider() {
    return const FDivider(
      color: Colors.white,
      height: 0.5,
      endIndent: 2,
    );
  }

  void _onMeasureRerenderReady(Object sender, void e) {
    if (mounted) {
      setState(() {
        _updateOutputs();
        _autoScrollToBottom();
      });
    }
  }

  void _updateOutputs() {
    outputs.clear();

    for (var item in application.measureItems) {
      for (var feature in item.measuredFeatures) {
        (feature as MeasureItemFeature).hostVisualArea?.mode;
      }
    }
    for (var item in application.measureItems) {
      if (item.calculator != null) {
        // 添加历史测量值
        outputs.addAll(item.calculator!.outputs);

        if (item.feature?.isActive == true && item.calculator!.output != null) {
          // 添加活动测量值
          outputs.add(item.calculator!.output!);
        }
      }
    }
    _checkIdStrLength();
  }

  void _checkIdStrLength() {
    idStrLength = outputs.length.toString().length;
  }

  String _getIdDisplayStr(int id) {
    return id.toString().padRight(idStrLength, ' ');
  }

  void _autoScrollToBottom() {
    if (outputs.isEmpty) return;
    Future.delayed(const Duration(milliseconds: 200), () {
      if (_scrollController.hasClients) {
        _scrollController.jumpTo(_scrollController.position.maxScrollExtent);
      }
    });
  }
}
